@@ -4,6 +4,7 @@ use crate::codec::{AsyncRead, AsyncWrite};
 use crate::connect::{Connect as TcpConnect, Connector as TcpConnector};
 use crate::http::{Protocol, Uri};
 use crate::service::{apply_fn, boxed, Service};
+use crate::time::{Millis, Seconds};
 use crate::util::timeout::{TimeoutError, TimeoutService};
 use crate::util::{Either, Ready};
 
@@ -29,18 +30,18 @@ type BoxedConnector =
 /// construction that finishes by calling the `.finish()` method.
 ///
 /// ```rust,no_run
-/// use std::time::Duration;
 /// use ntex::http::client::Connector;
+/// use ntex::time::Millis;
 ///
 /// let connector = Connector::default()
-///      .timeout(Duration::from_secs(5))
+///      .timeout(Millis(5_000))
 ///      .finish();
 /// ```
 pub struct Connector {
-    timeout: Duration,
+    timeout: Millis,
     conn_lifetime: Duration,
     conn_keep_alive: Duration,
-    disconnect_timeout: Duration,
+    disconnect_timeout: Millis,
     limit: usize,
     connector: BoxedConnector,
     ssl_connector: Option<BoxedConnector>,
@@ -64,10 +65,10 @@ impl Connector {
                     .map_err(ConnectError::from),
             ),
             ssl_connector: None,
-            timeout: Duration::from_secs(1),
+            timeout: Millis(1_000),
             conn_lifetime: Duration::from_secs(75),
             conn_keep_alive: Duration::from_secs(15),
-            disconnect_timeout: Duration::from_millis(3000),
+            disconnect_timeout: Millis(3_000),
             limit: 100,
         };
 
@@ -83,12 +84,24 @@ impl Connector {
         }
         #[cfg(all(not(feature = "openssl"), feature = "rustls"))]
         {
+            use rust_tls::{OwnedTrustAnchor, RootCertStore};
+
             let protos = vec![b"h2".to_vec(), b"http/1.1".to_vec()];
-            let mut config = ClientConfig::new();
-            config.set_protocols(&protos);
-            config
-                .root_store
-                .add_server_trust_anchors(&webpki_roots::TLS_SERVER_ROOTS);
+            let mut cert_store = RootCertStore::empty();
+            cert_store.add_server_trust_anchors(
+                webpki_roots::TLS_SERVER_ROOTS.0.iter().map(|ta| {
+                    OwnedTrustAnchor::from_subject_spki_name_constraints(
+                        ta.subject,
+                        ta.spki,
+                        ta.name_constraints,
+                    )
+                }),
+            );
+            let mut config = ClientConfig::builder()
+                .with_safe_defaults()
+                .with_root_certificates(cert_store)
+                .with_no_client_auth();
+            config.alpn_protocols = protos;
             conn.rustls(Arc::new(config))
         }
         #[cfg(not(any(feature = "openssl", feature = "rustls")))]
@@ -99,10 +112,12 @@ impl Connector {
 }
 
 impl Connector {
-    /// Connection timeout, i.e. max time to connect to remote host including dns name resolution.
+    /// Connection timeout.
+    ///
+    /// i.e. max time to connect to remote host including dns name resolution.
     /// Set to 1 second by default.
-    pub fn timeout(mut self, timeout: Duration) -> Self {
-        self.timeout = timeout;
+    pub fn timeout<T: Into<Millis>>(mut self, timeout: T) -> Self {
+        self.timeout = timeout.into();
         self
     }
 
@@ -129,14 +144,14 @@ impl Connector {
     #[cfg(feature = "rustls")]
     /// Use rustls connector for secured connections.
     pub fn rustls(self, connector: Arc<ClientConfig>) -> Self {
-        use crate::connect::rustls::{RustlsConnector, Session};
+        use crate::connect::rustls::RustlsConnector;
 
         const H2: &[u8] = b"h2";
         self.secure_connector(RustlsConnector::new(connector).map(|sock| {
             let h2 = sock
                 .get_ref()
                 .1
-                .get_alpn_protocol()
+                .alpn_protocol()
                 .map(|protos| protos.windows(2).any(|w| w == H2))
                 .unwrap_or(false);
             if h2 {
@@ -162,8 +177,8 @@ impl Connector {
     /// the delay between repeated usages of the same connection
     /// exceeds this period, the connection is closed.
     /// Default keep-alive period is 15 seconds.
-    pub fn keep_alive(mut self, dur: Duration) -> Self {
-        self.conn_keep_alive = dur;
+    pub fn keep_alive(mut self, dur: Seconds) -> Self {
+        self.conn_keep_alive = dur.into();
         self
     }
 
@@ -172,8 +187,8 @@ impl Connector {
     /// Connection lifetime is max lifetime of any opened connection
     /// until it is closed regardless of keep-alive period.
     /// Default lifetime period is 75 seconds.
-    pub fn lifetime(mut self, dur: Duration) -> Self {
-        self.conn_lifetime = dur;
+    pub fn lifetime(mut self, dur: Seconds) -> Self {
+        self.conn_lifetime = dur.into();
         self
     }
 
@@ -184,9 +199,9 @@ impl Connector {
     ///
     /// To disable timeout set value to 0.
     ///
-    /// By default disconnect timeout is set to 3000 milliseconds.
-    pub fn disconnect_timeout(mut self, dur: Duration) -> Self {
-        self.disconnect_timeout = dur;
+    /// By default disconnect timeout is set to 3 seconds.
+    pub fn disconnect_timeout<T: Into<Millis>>(mut self, timeout: T) -> Self {
+        self.disconnect_timeout = timeout.into();
         self
     }
 
@@ -263,7 +278,7 @@ impl Connector {
 
 fn connector(
     connector: BoxedConnector,
-    timeout: Duration,
+    timeout: Millis,
 ) -> impl Service<
     Request = Connect,
     Response = (Box<dyn Io>, Protocol),
