@@ -1,13 +1,11 @@
 use std::cell::{Cell, RefCell};
 use std::sync::{Arc, Mutex};
 use std::task::{Context, Poll, Waker};
-use std::{cmp, fmt, io, mem, pin::Pin, time};
-
-use bytes::BytesMut;
+use std::{cmp, fmt, io, mem, pin::Pin};
 
 use crate::codec::{AsyncRead, AsyncWrite, ReadBuf};
-use crate::rt::time::sleep;
-use crate::util::poll_fn;
+use crate::time::{sleep, Millis};
+use crate::util::{poll_fn, BytesMut};
 
 #[derive(Default)]
 struct AtomicWaker(Arc<Mutex<RefCell<Option<Waker>>>>);
@@ -171,7 +169,7 @@ impl Io {
             remote.read = IoState::Close;
             remote.waker.wake();
         }
-        sleep(time::Duration::from_millis(35)).await;
+        sleep(Millis(35)).await;
     }
 
     /// Add extra data to the remote buffer and notify reader
